MCA <- function(vary, varx, lon = NULL, lat = NULL, neofs = 15,
                corr = FALSE, siglev = 0.98, weight = TRUE) {
  # Checking vary
  if (!is.numeric(vary) || !is.array(vary)) {
    stop("Parameter 'vary' must be a numeric array.")
  }
  if (length(dim(vary)) != 3) {
    stop("'vary' must have dimensions c(n. start dates/forecast time steps/time samples, n. latitudes, n. longitudes).")
  }

  # Checking varx
  if (!is.numeric(varx) || !is.array(varx)) {
    stop("Parameter 'varx' must be a numeric array.")
  }
  if (length(dim(varx)) != 3) {
    stop("'varx' must have dimensions c(n. start dates/forecast time steps/time samples, n. latitudes, n. longitudes).")
  }

  if (weight) {
    # Checking lon and lat
    if (!is.numeric(lon) || !is.numeric(lat)) {
      stop("'lon' and 'lat' must be numeric vectors.")
    }
    if (any(lon > 360 | lon < -360)) {
      .warning("some 'lon's out of the range [-360, 360].")
    }
    if (any(lat > 90 | lat < -90)) {
      stop("'lat' must contain values within the range [-90, 90].")
    }
    nlon <- length(lon)
    nlat <- length(lat)
  }

  # Checking neofs
  if (!is.numeric(neofs)) {
    stop("'neofs' must be numeric.")
  }
  neofs <- round(neofs)

  # Checking corr
  if (!is.logical(corr)) {
    stop("'corr' must be either TRUE or FALSE.")
  }

  # Checking siglev
  if (!is.numeric(siglev)) {
    stop("'siglev' must be numeric.")
  }
  if (siglev < 0 || siglev > 1) {
    stop("'siglev' must be in the range [0, 1].")
  }

  # Checking weight
  if (!is.logical(weight)) {
    stop("'weight' must be either TRUE or FALSE.")
  }

  dx <- dim(varx)
  #if (length(dx) == 3) {
    nlonx <- dx[3]
    nlatx <- dx[2]
    nsx <- nlonx * nlatx
    nt <- dx[1]
  #} else {
  #  nlonx <- 1
  #  nlatx <- dx[2]
  #  nsx <- nlonx * nlatx
  #  nt <- dx[1]
  #}

  # Security check
  if (nlatx != nlat) {
    stop("Inconsistent number of latitudes and input field dimensions.")
  }
  if (nlonx != nlon) {
    stop("Inconsistent number of longitudes and input field dimensions.")
  }

  if (weight) {
    # Area weighting. Weights for EOF; needed to compute the
    # fraction of variance explained by each EOFs
    wght <- array(cos(lat * pi/180), dim = (c(nlat, nlon)))
    
    # We want the covariance matrix to be weigthed by the grid
    # cell area so the anomaly field is weighted by its square
    # root since the covariance matrix equals transpose(ano)
    # times ano.
    wght <- sqrt(wght)
    varx <- varx * InsertDim(wght, 1, nt)
  }

  # The use of the correlation matrix is done under the option
  # corr.
  if (corr == TRUE) {
    stdv <- apply(varx, sd, c(2, 3), na.rm = TRUE)
    varx <- varx/InsertDim(stdv, 1, nt)
  }

  X <- varx # ntime, nlat, nlon
  dim(X) <- c(nt, nsx)
  x_na <- apply(X, 1, function(time_series) if (any(is.na(time_series)) {TRUE} else {FALSE}))
  Xyes <- X[, which(x_na)] # ntime, nspace_reduced
  
  dy <- dim(vary)
  #if (length(dy) == 3) {
    nlony <- dy[3]
    nlaty <- dy[2]
    nsy <- nlony * nlaty
    nt <- dy[1]
  #} else {
  #  nlony <- 1
  #  nlaty <- dy[2]
  #  nsy <- nlony * nlaty
  #  nt <- dy[1]
  #}

  # Security check
  if (nlatx != nlat) {
    stop("Inconsistent number of latitudes and input field dimensions.")
  }
  if (nlonx != nlon) {
    stop("Inconsistent number of longitudes and input field dimensions.")
  }

  if (weight) {
    vary <- vary * InsertDim(wght, 1, nt)
  }

  # The use of the correlation matrix is done under the option
  # corr.
  if (corr == TRUE) {
    stdv <- apply(vary, sd, c(2, 3), na.rm = TRUE)
    vary <- vary/InsertDim(stdv, 1, nt)
  }

  Y <- vary # time, lat, lon
  dim(Y) <- c(nt, nsy)
  y_na <- apply(Y, 1, function(time_series) if (any(is.na(time_series)) {TRUE} else {FALSE}))
  Yyes <- Y[, which(y_na)] # time, space_reduced
  
  # covariance matrix
  
  covXY <- (t(Xyes) %*% Yyes)/(nt - 1)
  
  # MCA analysis
  
  MCA <- svd(covXY)
  
  # outputs
  
  nm <- neofs #number of modes to be retained
  
  SV <- MCA$d  #singular values
  SV2 <- SV * SV
  S <- sum(SV2)
  SCF <- SV2/S  #squared covariance fraction
  SCF <- SCF[1:nm] * 100
  
  Ux <- MCA$u
  Vy <- MCA$v
  Un <- Ux[, 1:nm]
  Vn <- Vy[, 1:nm]
  ECu <- t(Xyes %*% Un)
  ECv <- t(Yyes %*% Vn)  #(nm, nt)
  ECu_std <- array(dim = c(nm, nt))
  ECv_std <- array(dim = c(nm, nt))
  for (i in 1:nm) {
    Su <- sd(ECu[i, ])
    ECu_std[i, ] <- ECu[i, ]/Su
    Sv <- sd(ECv[, i])
    ECv_std[i, ] <- ECv[i, ]/Sv
  }
  
  RUV <- array(dim = nm)
  for (i in 1:nm) {
    RUV[i] <- cor(ECu_std[i, ], ECv_std[i, ])
  }
 
  MCAU <- array(dim = c(nm, nlatx, nlonx))
  MCAV <- array(dim = c(nm, nlaty, nlony))
  for (m in nm) {
    MCAU[m, , ] <- apply(X, c(2, 3), 
      function(time_series) {
        if (any(is.na(time_series))) {
          NA
        } else {
          lm(time_series ~ ECu_std[m, ])$coefficients[[2]]
        }
      })
    MCAV[m, , ] <- apply(Y, c(2, 3),
      function(time_series) {
        if (any(is.na(time_series))) {
          NA
        } else {
          lm(time_series ~ ECv_std[m, ])$coefficients[[2]]
        }
      })
  }

  if (weight) {
    FALTA DIVIDIR POR LOS wght!!!!
  }
  
  # t-Test
  
  cl <- siglev
  cl2 <- (1 - cl)/2
  t <- qt(cl + cl2, nt - 2)
  R <- sqrt((t * t)/((t * t) + nt - 2))
  
  invisible(list(SC = SV2, SCFs = SCF, RUVs = RUV, ECs_U = ECu_std, 
                 MCAs_U = MCAU, ECs_V = ECv_std, MCAs_V = MCAV, 
                 Rtest = R)) 
  #               CORRs_U = CORRU, CORRs_V = CORRV))
}
